%{ open Ast %}

%token SEMI LBRACE RBRACE LPAREN RPAREN COMMA LBRACK RBRACK LARRAY RARRAY
%token TIMES DIVIDE INCR DECR
%token PLUS MINUS 
%token GT LT EQ NOTEQ GTEQ LTEQ
%token NOT AND OR
%token DOT ASSIGN
%token INT BOOLEAN STRING GRID ARRAY
%token IF ELSE WHILE FOR RETURN
%token VISIBLE REPEATABLE
%token CELL ROW COLUMN DIAGNAL STACK BAND BLOCK
%token SUMROW SUMCOL SUMDIAG SUMBLOCK MAXROW MAXCOL MAXDIAG MAXBLOCK MINROW MINCOL MINDIAG MINBLOCK ISVISIBLE CHECKGRID
%token LEFT RIGHT
%token <bool> TRUE FALSE
%token <int> LITERAL
%token <string> STRINGLITERAL
%token <string> ID
%token EOF

%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
%nonassoc NOT
%left AND OR
%left EQ NOTEQ
%left LT GT LTEQ GTEQ 
%left PLUS MINUS
%left TIMES DIVIDE
%left INCR DECR           
%left BOOLEAN INT STRING GRID ARRAY

%start program
%type <Ast.program> program

%%

program:
	/* nothing*/	{ [],[] }
	| program vdecl { ($2::fst $1), snd $1 }
	| program fdecl { fst $1, ($2 :: snd $1) }

vdecl:
INT ID SEMI	{ {vtype=Integer;vname=$2} }
| STRING ID SEMI	{ {vtype=Strings;vname=$2} }
| BOOLEAN ID SEMI	{ {vtype=Boolean;vname=$2} }
| ARRAY ID SEMI	{ {vtype=Arrays;vname=$2} }

fdecl:
	ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list RBRACE
		{{ fname=$1;
		 formals=$3;
		 locals=List.rev $6;
		 body=List.rev $7}}

formal:
	INT ID	{ {ftype=Integer;foname=$2} }
	| STRING ID	{ {ftype=Strings;foname=$2} }
	| BOOLEAN ID	{{ftype=Boolean;foname=$2} }
	| ARRAY ID	{ {ftype=Arrays;foname=$2}}

formals_opt:
	/*nothing*/	{[]}
	|formal_list {List.rev $1}

formal_list:
	formal			{[$1]}
	|formal_list COMMA formal	{$3::$1}


vdecl_list:
	/*nothing*/	{[]}
	|vdecl_list vdecl {$2::$1}

	
stmt_list:
/* nothing */ { [] }
| stmt_list stmt { $2 :: $1 }

stmt:
  expr SEMI { Expr($1) }
| LBRACE stmt_list RBRACE	{ Block(List.rev $2) }
| IF LPAREN expr RPAREN stmt  %prec NOELSE { IfElse($3, $5, Block([])) }
| IF LPAREN expr RPAREN stmt ELSE stmt { IfElse($3, $5, $7) }
| WHILE LPAREN expr RPAREN stmt { While($3, $5) }
| FOR LPAREN expr SEMI expr SEMI expr RPAREN stmt { For($3, $5, $7, $9) }
| RETURN expr_opt	SEMI { Return($2) }

expr_opt:
/* nothing */ { Noexpr }
| expr { $1 }

expr:
/*Types: */ 
  LITERAL { Literal($1) }
| TRUE { Bool(true) }
| FALSE	{ Bool(false) }
| STRINGLITERAL { String($1) }
| LARRAY literal_array RARRAY { Array(List.rev $2) }
| ID {Id($1)}
| ID LBRACK expr RBRACK	{Array1($1, $3)}

/*Arithmetic: */
| expr PLUS expr { Binop($1, Add, $3) }
| expr MINUS expr { Binop($1, Sub, $3) }
| expr TIMES expr { Binop($1, Mult, $3) }
| expr DIVIDE expr { Binop($1, Div, $3) }

/* Assignment */
| expr ASSIGN expr { Assign($1, $3) }
| expr INCR { Assign($1, Binop($1, Add, Literal(1)))}
| expr DECR { Assign($1, Binop($1, Sub, Literal(1)))}

/*Comparison: */
| expr GT expr { Binop($1, Greater, $3) }
| expr LT expr { Binop($1, Less, $3) }
| expr EQ expr { Binop($1, Equal, $3) }
| expr NOTEQ expr { Binop($1, Neq, $3) }
| expr GTEQ expr { Binop($1, Geq, $3) }
| expr LTEQ expr { Binop($1, Leq, $3) }

/*Logical: */
| NOT expr { Not($2) }
| expr AND expr { Binop($1, And, $3) }
| expr OR expr { Binop($1, Or, $3) }

/*Something about Grid*/
| GRID DOT CELL LPAREN expr COMMA expr RPAREN	{ Cell($5,$7) }
/*	| GRID DOT ROW LPAREN expr RPAREN	{ Row($5) }
	| GRID DOT COLUMN LPAREN expr RPAREN { Column($5) }
	| GRID DOT DIAGNAL LPAREN LEFT RPAREN { Diagnal_left }
	| GRID DOT DIAGNAL LPAREN RIGHT RPAREN { Diagnal_right }
	| GRID DOT BLOCK LPAREN expr COMMA expr RPAREN { Block_grid($5,$7) }
	| GRID DOT BAND LPAREN expr RPAREN { Band($5) }
	| GRID DOT STACK LPAREN expr RPAREN { Stack($5) }*/
/*| GRID DOT CELL LPAREN expr COMMA expr RPAREN DOT SETASGIVEN	LPAREN expr RPAREN{ SetAsGiven($5,$7,$12) }*/
/*| GRID DOT ISREPEATABLE	LPAREN expr RPAREN{ IsRepeatable($5) }*/
| GRID DOT VISIBLE	LPAREN expr COMMA expr RPAREN{ IsVisible($5,$7) }
/*| GRID DOT DEFINEBLOCK LPAREN expr COMMA expr COMMA expr COMMA expr RPAREN { DefineBlock($5,$7, $9,$11) }*/
| GRID DOT SUMROW LPAREN expr RPAREN	{ SumRow($5) }
| GRID DOT SUMCOL LPAREN expr RPAREN	{ SumCol($5) }
| GRID DOT SUMDIAG LPAREN expr RPAREN {	SumDiag($5) }
| GRID DOT SUMBLOCK LPAREN expr RPAREN	{ SumBlock($5) }
| GRID DOT MAXROW LPAREN expr RPAREN	{ MaxRow($5) }
| GRID DOT MAXCOL LPAREN expr RPAREN	{ MaxCol($5) }
| GRID DOT MAXDIAG LPAREN expr RPAREN	{ MaxDiag($5) }
| GRID DOT MAXBLOCK LPAREN expr RPAREN { MaxBlock($5) }
| GRID DOT MINROW LPAREN expr RPAREN	{ MinRow($5) }
| GRID DOT MINCOL LPAREN expr RPAREN	{ MinCol($5) }
| GRID DOT MINDIAG LPAREN expr RPAREN	{ MinDiag($5) }
| GRID DOT MINBLOCK LPAREN expr RPAREN { MinBlock($5) }
| GRID DOT CHECKGRID LPAREN RPAREN { CheckGrid }


/* Call functions*/
| ID LPAREN actuals_opt RPAREN { Call($1,$3)}


literal_array:
	LITERAL 	{ [$1] }
	|literal_array COMMA LITERAL	{ $3::$1 }
	

actuals_opt:
	/*nothing*/	{ [] }
	| actual_list {List.rev $1}

actual_list:
	expr	{ [$1] }
	| actual_list COMMA expr	{ $3::$1 } 
