open Ast
open Bytecode

(* Stack layout just after "Ent":

              <-- SP
   Local n
   ...
   Local 0
   Saved FP   <-- FP
   Saved PC
   Arg 0
   ...
   Arg n *)

let execute_prog prog =
  let stack = Array.make 20000 ["0"]
  and globals = Array.make prog.num_globals ["0"]
  and grid = Array.make 111 (0,false) 
  in

  let rec exec fp sp pc = match prog.text.(pc) with
    Lit i  -> stack.(sp) <- [string_of_int i] ; exec fp (sp+1) (pc+1)
  | Lits j -> stack.(sp) <- [j] ; exec fp (sp+1) (pc+1)
  | Lita a -> stack.(sp) <- List.map string_of_int a ; exec fp (sp+1) (pc+1)
  | Drp    -> exec fp (sp-1) (pc+1)
  | Bin op -> let op1 = (if ( int_of_string (List.hd stack.(sp-2))=(-1) ) then ( int_of_string (List.hd stack.(sp-3)) )
                   else ( int_of_string (List.hd stack.(sp-2)) ) )                     
		   and op2 =int_of_string (List.hd  stack.(sp-1)) in                   
		   stack.(sp-2) <- [string_of_int (let boolean i = if i then 1 else 0 in
              match op with
	Add     -> op1 + op2
      | Sub     -> op1 - op2
      | Mult    -> op1 * op2
      | Div     -> op1 / op2
      | Or      -> let a=(if op1=0 then false else true) and b=(if op2=0 then false else true) in boolean (a || b)
      | And     -> let a=(if op1=0 then false else true) and b=(if op2=0 then false else true) in boolean (a && b)
      | Equal   -> boolean (op1 =  op2)
      | Neq     -> boolean (op1 != op2)
      | Less    -> boolean (op1 <  op2)
      | Leq     -> boolean (op1 <= op2)
      | Greater -> boolean (op1 >  op2)
      | Geq     -> boolean (op1 >= op2))] ;
      exec fp (sp-1) (pc+1)
      | Nota -> if int_of_string (List.hd stack.(sp-1))=0 then stack.(sp-1) <- ["1"]
		else stack.(sp-1) <- ["0"] ; exec fp sp (pc+1)
(*row*) 
  | Grow   -> let t= let row = int_of_string(List.hd(stack.(sp-1))) in
                            (let rec range a b = 
                                if b > 8 then []
                                else  string_of_int(fst(grid.(a)))::range (a+1) (b+1)
                            in range (30+9*(row-1)) 0 ) in stack.(sp)<-t; 
                            exec fp (sp+1) (pc+1) 
  | Srow   -> let row = int_of_string(List.hd(stack.(sp-1))) in
              let rec store n = function
                      []  ->  ()
                    | hd::tl -> grid.((row-1)*9+30+n) <- (int_of_string(hd),false); store (n+1) tl
              in store 0 (stack.(sp-2)) ; exec fp (sp+1) (pc+1)
              
(*column*)
  | Gcol   -> let t = let col = int_of_string(List.hd(stack.(sp-1))) in
              (let rec range a b = 
                  if b < 9 then string_of_int(fst(grid.(a)))::range (a+9) (b+1)
                  else  []
              in range ((col-1)+30) 0) in stack.(sp) <- t; exec fp (sp+1) (pc+1)       
  | Scol   -> let col = int_of_string(List.hd(stack.(sp-1))) in
              let rec store n = function 
                    []  -> ()
                  | hd::tl -> grid.((col-1)+30+n*9) <- (int_of_string(hd),false); store (n+1) tl 
              in store 0 (stack.(sp-2))  ; exec fp (sp+1) (pc+1) 
    | Setv -> let row = (if List.hd stack.(sp-2)="-1" then (int_of_string (List.hd stack.(sp-3))-1) else (int_of_string (List.hd stack.(sp-2)))-1) and col=(int_of_string (List.hd stack.(sp-1))-1) in 
              let num = fst (grid.(30+9*row+col)) in  ((*print_endline (string_of_int (30+9*row+col)); *)grid.(30+9*row+col)<-(num,true)); exec fp sp (pc+1)
    | Lod i -> if int_of_string (List.hd stack.(sp-1)) = -1 then stack.(sp) <- globals.(i)
		else stack.(sp) <- [List.nth globals.(i) ((int_of_string (List.hd stack.(sp-1)))-1)] ; exec fp (sp+1) (pc+1)
  | Str i   -> if List.hd stack.(sp-1) = "-1" then globals.(i) <- stack.(sp-2)
		else (let atemp=Array.of_list globals.(i) in 
               atemp.(int_of_string (List.hd stack.(sp-1))-1)<-List.hd stack.(sp-3);
               let ltemp=Array.to_list atemp in  globals.(i)<- ltemp) ; exec fp sp (pc+1)

  | Lfp i -> if int_of_string (List.hd stack.(sp-1))= -1 then stack.(sp) <- stack.(fp+i)
		else stack.(sp) <- [List.nth stack.(fp+i) (int_of_string (List.hd stack.(sp-1)))] ; exec fp (sp+1) (pc+1)

  | Sfp i   ->  if List.hd stack.(sp-1) = "-1" then stack.(fp+i) <- stack.(sp-2)
		else (let atemp=Array.of_list (stack.(fp+i)) in 
                atemp.(int_of_string (List.hd stack.(sp-1))-1)<-(List.hd stack.(sp-3)); 
                let ltemp=Array.to_list atemp in stack.(fp+i)<- ltemp)  ; exec fp sp (pc+1)
  | Lodg    ->  if List.hd stack.(sp-2) = "-1" then stack.(sp) <- [string_of_int (fst (grid.(30+9*(int_of_string (List.hd stack.(sp-3))-1)+(int_of_string (List.hd stack.(sp-1))-1))))] 
            else stack.(sp) <- [string_of_int (fst (grid.(30+9*((int_of_string (List.hd stack.(sp-2)))-1)+(int_of_string (List.hd stack.(sp-1)))-1)))]; exec fp (sp+1) (pc+1)
  | Strg i  ->  if (List.hd stack.(sp-2) = "-1") then grid.(30+9*(int_of_string (List.hd stack.(sp-3))-1)+(int_of_string (List.hd stack.(sp-1)))-1)<- if (List.hd stack.(sp-4) = "-1") then (int_of_string (List.hd stack.(sp-5)), i) 
                             else (int_of_string (List.hd stack.(sp-4)), i) 
             else (if (List.hd stack.(sp-3) = "-1") then grid.(30+9*((int_of_string (List.hd stack.(sp-2))-1))+(int_of_string (List.hd stack.(sp-1))-1))<-(int_of_string (List.hd stack.(sp-4)), i)
             else grid.(30+9*((int_of_string (List.hd stack.(sp-2))-1))+(int_of_string (List.hd stack.(sp-1))-1))<-(int_of_string (List.hd stack.(sp-3)), i)) ;exec fp sp (pc+1)
  | Jsr(-1) ->  if List.length (stack.(sp-1))=1 
	        then print_endline (List.hd stack.(sp-1))
	        else print_endline("[|" ^ String.concat "," stack.(sp-1) ^ "|]") ; 
	exec fp sp (pc+1)
  | Jsr(-2) ->  stack.(sp) <- [string_of_int (Random.self_init ();Random.int 9)] ; exec fp (sp+1) (pc+1) 
  | Jsr(-3) ->  for i=0 to 8 do 
		 for j=0 to 8 do
		   if snd (grid.(30+i*9+j))=true then print_string (string_of_int(fst grid.(30+i*9+j))^" ")(*print_int (fst grid.(30+i*9+j))*)
			else print_string "* "(*print_int (fst grid.(30+i*9+j))*) 
		 done;
		 print_endline " "
	        done ; exec fp sp (pc+1)
  | Ran    ->  stack.(sp) <- [string_of_int (Random.self_init (); Random.int ( int_of_string (List.hd stack.(sp-1)))+1)]; exec fp (sp+1) (pc+1) 
  | Chkg   ->  if snd grid.(0)=true then for i=0 to 8 do
	                                    let rec sum b = if b<9 then (sum (b+1))+(fst grid.(30+i*9+b)) else 0 in let a=(sum 0) in if a != fst grid.(0) then print_endline ("Warning: Sum of row "^string_of_int (i+1)^" = " ^string_of_int a^", breaks the sum of row rule!") else ()
					      done
					else ();(*Check the sum of row rule*) 

	       if snd grid.(1)=true then for i=0 to 8 do 
				            let rec sum b = if b<9 then (sum (b+1))+(fst grid.(30+b*9+i)) else 0 in let a=(sum 0) in if a != fst grid.(1) then print_endline ("Warning: Sum of column "^string_of_int (i+1)^" = "^string_of_int a^", breaks the sum of column rule!") else ()
					      done
					else ();(*Check the sum of column rule*)

	       if snd grid.(2)=true then 
		                           let rec sum b = if b<9 then (sum (b+1))+(fst grid.(30+b*9+b)) else 0 in let a = (sum 0) in if a != fst grid.(2) then print_endline "Left diagnal breaks the sum of diagnal rule" else ();
                                           let rec sums c = if c<9 then (sums (c+1))+(fst grid.(30+9*c+8-c)) else 0 in let b = (sums 0) in if b != fst grid.(2) then print_endline "Right diagnal breaks the sum of diagnal rule" else()
                                       else();(*Check the sum of diagnal rule*)

	      if snd grid.(3)=true then for i=0 to 8 do
		                          let rec max a b= if a<9 then (if b<(fst grid.(30+i*9+a)) then max (a+1) (fst grid.(30+i*9+a)) else max (a+1) b) else b in let c=max 0 0 in
					if c > fst grid.(3) then print_endline ("Row "^string_of_int (i+1)^" breaks the maximum of row rule") else ()
                                             done  
                                          else ();(*Check the max of row rule*)

	      if snd grid.(4)=true then for i=0 to 8 do 
		                         let rec max a b= if a<9 then (if b<(fst grid.(30+a*9+i)) then max (a+1) (fst grid.(30+a*9+i)) else max (a+1) b) else b in let c=max 0 0 in
					if c > fst grid.(4) then print_endline ("Column "^string_of_int (i+1)^" breaks the maximum of column rule") else ()
done else (); (*Check the max of column rule*)

	      if snd grid.(5)=true then let rec max a b = if a<9 then (if b<(fst grid.(30+a*9+a)) then max (a+1) (fst grid.(30+a*9+a)) else max (a+1) b) else b in let c= max 0 0 in if c> fst grid.(5) then print_endline "Left diagnal breaks the maximum of diagnal rule" else ();
				  let rec maxr a b = if a<9 then (if b<(fst grid.(30+a*9+8-a)) then maxr (a+1) (fst grid.(30+a*9+8-a)) else maxr (a+1) b) else b in let c= maxr 0 0 in if c> fst grid.(5) then print_endline "Right diagnal breaks the maximum of diagnal rule" else ()
else (); (*Check the max of diagnal rule*)
	
	     if snd grid.(6)=true then for i=0 to 8 do
		                  let rec min a b= if a<9 then (if b>(fst grid.(30+i*9+a)) then min (a+1) (fst grid.(30+i*9+a)) else min (a+1) b) else b in let c=min 0 1000 in
					if c < fst grid.(6) then print_endline ("Row "^string_of_int (i+1)^" breaks the minimum of row rule") else ()
done else ();(*Check the min of row rule*)

	     if snd grid.(7)=true then for i=0 to 8 do 
		                  let rec min a b= if a<9 then (if b>(fst grid.(30+a*9+i)) then min (a+1) (fst grid.(30+a*9+i)) else min (a+1) b) else b in let c=min 0 1000 in
					if c < fst grid.(7) then print_endline ("Column "^string_of_int (i+1)^" breaks the minimum of column rule") else ()
done else (); (*Check the min of column rule*)

	      if snd grid.(8)=true then let rec min a b = if a<9 then (if b>(fst grid.(30+a*9+a)) then min (a+1) (fst grid.(30+a*9+a)) else min (a+1) b) else b in let c= min 0 1000 in if c> fst grid.(8) then print_endline "Left diagnal breaks the maximum of diagnal rule" else ();
				  let rec minr a b = if a<9 then (if b>(fst grid.(30+a*9+8-a)) then minr (a+1) (fst grid.(30+a*9+8-a)) else minr (a+1) b) else b in let c= minr 0 1000 in if c> fst grid.(8) then print_endline "Right diagnal breaks the maximum of diagnal rule" else ()
else (); (*Check the min of diagnal rule*)
	exec fp sp (pc+1)
  | Jsr i   -> stack.(sp)   <- [string_of_int (pc + 1)]       ; exec fp (sp+1) i
  | Ent i   -> stack.(sp)   <- [string_of_int fp]           ; exec sp (sp+i+1) (pc+1)
  | Rts i   -> let new_fp =int_of_string (List.hd stack.(fp)) and new_pc = int_of_string (List.hd stack.(fp-1)) in
               stack.(fp-i-1) <- stack.(sp-1) ; exec new_fp (fp-i) new_pc
  | Beq i   -> exec fp (sp-1) (pc + if int_of_string (List.hd stack.(sp-1)) =  0 then i else 1)
  | Bne i   -> exec fp (sp-1) (pc + if int_of_string (List.hd stack.(sp-1)) != 0 then i else 1)
  | Bra i   -> exec fp sp (pc+i)
  | Hlt     -> ()

  in exec 0 0 0


