{ 
  open Parser
}

rule token = parse 
  [' ' '\t' '\n' '\r'] {token lexbuf} (*White spaces*)
| "/*" {comment lexbuf}               


(* Groupinp *)
| "begin"    {LBRACE}
| "end"      {RBRACE}
| '('      {LPAREN}	
| ')'      {RPAREN}
| '['	   {LBRACK}
| ']'	   {RBRACK}
| "[|"	   {LARRAY}
| "|]"	   {RARRAY}
(* Operators: *)
  (* Arithmetic: *)
| '+'      {PLUS}
| '-'      {MINUS}
| '*'      {TIMES}
| '/' 		 {DIVIDE}
| "++"     {INCR}
| "--"     {DECR}
   (* Comparison: *)
| '>'      {GT}
| '<'      {LT}
| '='      {EQ}
| "!="     {NOTEQ}
| ">="     {GTEQ}
| "<="     {LTEQ}
   (* Logical: *)
| "not"    {NOT}
| "and"    {AND}
| "or"     {OR}
(* Other: *)
| '.'      {DOT}
| ','      {COMMA}
| ';'      {SEMI}
| '~'     {ASSIGN}
(* Types *)
| "int"    {INT}
| "bool"   {BOOLEAN}
| "string" {STRING}
| "Array"  {ARRAY}
| "Grid"   {GRID}
| "Cell"   {CELL}
| "Row"    {ROW}
| "CheckGrid" {CHECKGRID}
| "Random"  {RANDOM}
| "Column" {COLUMN}
(* Keywords: *)
| "if"     {IF}
| "else"   {ELSE}
| "for"    {FOR}
| "while"  {WHILE}
| "true"   {TRUE(true)}
| "false"  {FALSE(false)}
| "return" {RETURN}
(*give a sample sentence*)
| "MakeVisible"    {VISIBLE}    (*MakeVisible*)
(*give a sample sentence*)
| "SumOfRows"      {SUMROW}     (*The total that all the values in each row must sum to.*)
| "SumOfColumns"      {SUMCOL}     (*The total that all the values in each column must sum to.*)
| "SumOfDiagnals"     {SUMDIAG}    (*The total that all the values in each diagonal must sum to.*)
| "MaxValueOfRows"         {MAXROW}     (*What the maximum value of each row is permitted to be.*)
| "MaxValueOfColumns"         {MAXCOL}     (*What the maximum value of each column is permitted to be.*)
| "MaxValueOfDiagnals"        {MAXDIAG}    (*What the maximum value of each diagonal is permitted to be.*)
| "MinValueOfRows"         {MINROW}     (*Do we still need this? what the minimum value of each row is permitted to be.*)
| "MinValueOfColumns"         {MINCOL}     (*Do we still need this? what the minimum value of each row is permitted to be.*)
| "MinValueOfDiagnals"        {MINDIAG}    (*Do we still need this? what the minimum value of each diagonal is permitted to be.*)
| ['0'-'9']+ as lit {LITERAL(int_of_string lit)}
| ['a'-'z' 'A'-'Z' '_']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lit { ID(lit) }
| '\"'[^ '\"']*'\"' as
	lxm { STRINGLITERAL(String.sub lxm 1 ((String.length lxm)-2)) }
| eof      {EOF}                      (*End of file*)
| _ as char { raise (Failure("illegal character "^Char.escaped char))}

| _ as char {
             let pos = lexbuf.Lexing.lex_curr_p in
                 raise (Failure("Illegal character: " ^ Char.escaped char
                 ^ " in line #" ^ (string_of_int pos.Lexing.pos_lnum))) }



and comment = parse 
  "*/" {token lexbuf}
| _ {comment lexbuf}
